unit dclublib;
{
  Filename:    DClubLib.pas
  Version:     4.0
  Last update: 2000-19-03
  Author:      Guido Festraets
  Copyright:   (c) 2000 Studiebureau Festraets
  Delphi:		   32-bit versions
  Platform:	   Windows 32-bit versions
  Description:
    A collection of functions and procedures for the 32 bits versions
    of Delphi, written for DelphiLand.
    While these routines are quite useful, my first objective was
    "educational". Therefore, the code is abundantly commented.
    The latest update can be obtained from the DelphiLand site
    http://www.festra.com/eng/download.htm as DLyymmdd.zip e.g. DL000618.zip
}

interface

uses Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
FILECTRL;

//--------- Date and time -----

type
  TMoveResult = (fmovOK, fmovNOTDELETED, fmovFAIL);

function Age(const BirthDate: TDate): integer;
function BirthDay(const BirthDate: TDate): TDate;
function DateInYear(const ADate: TDate; const InYear: integer): TDate;
function DateToISO(const ADate: TDate): string;
function DateToISOshort(const ADate: TDate): string;
function DateToYear(const ADate: TDate): integer;
function DaysInMonth(const AYear, AMonth: integer): integer;
procedure SetSysDateTime(const TheDateTime: TDateTime);
procedure SetSysTime(const TheTime: TDateTime);
procedure SetSysDate(const TheDate: TDateTime);

//--------- String -----

function IsEmptyStr(const S: string): Boolean;
function IsInStr(const SubStr, S: string): Boolean;
function LastChar(const S: string): Char;
function LeftStr(const S: string; const Len: integer): string;
function NotInStr(const SubStr, S: string): Boolean;
function RightStr(const S: string; const Len: integer): string;
function PadLeft(const S: string; const Len: integer; const Ch: char): string;
function PadRight(const S: string; const Len: integer; const Ch: char): string;
function RepliStr(const S: string; const Len: integer): string;
function ParseFirstRemoved(const S: string; const Delimiters: string): string;
function ParseGetFirst(const S: string; const Delimiters: string): string;
function ParseGetLast(const S: string; const Delimiters: string): string;
function ParseToList(S: string; const Delimiters: string): TStringList;

//--------- File and Directory -------

function AppDir: string;
function CreateDirForce(const NewDir: string): Boolean;
function FileCopy(const Source, Dest: string;
  const Overwrite: Boolean): Boolean;
function FileDateTime(const TheFile: string): TDateTime;
function FileDateTimeStr(const TheFile: string): string;
function FileMove(const Source, Dest: string;
  const Overwrite: Boolean): TMoveResult;

implementation

//--------- Date and time -----

// Return: the age of a person in years
function Age(const BirthDate: TDate): integer;
var
  ToDay: TDate;
begin
  Today := Date;
  Result := DateToYear(Today) - DateToYear(BirthDate);
  // ...minus 1 if Today is BEFORE the BirthDay
  if Today < BirthDay(BirthDate) then dec(Result);
end;

// Return: the date of BirthDate, translated to
// the actual year we are in.
// Birthdate of 29 Feb in a leapyear: is returned
// as March 1 if the current year is not a leapyear.
function BirthDay(const BirthDate: TDate): TDate;
begin
  // Simply "transpose" the BirthDate to "this year"
  Result := DateInYear(BirthDate, DateToYear(Date));
end;

// Return: a given date "transposed" to another year
// When transposing Feb. 29 of a leapyear to a non-leap year,
// March 1 is returned
function DateInYear(const ADate: TDate; const InYear: integer): TDate;
var
  Y, M, D: word;
begin
  DecodeDate(ADate, Y, M, D);
  if (D = 29) and (M = 2) and (not IsLeapYear(InYear)) then
    Result := EncodeDate(InYear, 3, 1)
  else
    Result := EncodeDate(InYear, M, D);
end;

// Return: string with the "ISO 8601" format
// of a given date, e.g. 2000-12-31
function DateToISO(const ADate: TDate): string;
var
  Y, M, D: word;
begin
  DecodeDate(ADate, Y, M, D);
  Result := IntToStr(Y) + '-' + PadLeft(IntToStr(M), 2, '0') + '-' +
    PadLeft(IntToStr(D), 2, '0');
end;

// Return: string with the alternative short "ISO 8601" format
// of a given date, e.g. 20001231
function DateToISOshort(const ADate: TDate): string;
var
  Y, M, D: word;
begin
  DecodeDate(ADate, Y, M, D);
  Result := IntToStr(Y) + PadLeft(IntToStr(M), 2, '0') +
    PadLeft(IntToStr(D), 2, '0');
end;
// Return: the number of days in a given month in a given year
function DaysInMonth(const AYear, AMonth: integer): integer;
const
  DaysPerMonth: array[1..12] of integer =
    (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
begin
  if (AMonth = 2) and IsLeapYear(AYear) then Result := 29
  else Result := DaysPerMonth[AMonth];
end;

// Return: the year of the date passed in ADate
function DateToYear(const ADate: TDate): integer;
var
  Year, Month, Day: word;
begin
  DecodeDate(ADate, Year, Month, Day);
  Result := Year;
end;

procedure SetSysDateTime(const TheDateTime: TDateTime);
var
  ST: TSystemTime;
begin
  with ST do begin
    DecodeDate(TheDateTime, wYear, wMonth, wDay);
    DecodeTime(TheDateTime, wHour, wMinute, wSecond, wMilliSeconds);
  end;
  if not SetLocalTime(ST) then
    raise Exception.Create(SysErrorMessage(GetLastError));
end;

procedure SetSysTime(const TheTime: TDateTime);
begin
  SetSysDateTime(TheTime + Date);
end;

procedure SetSysDate(const TheDate: TDateTime);
begin
  SetSysDateTime(Time + TheDate)
end;

//--------- File and Directory -------

// Return: the directory of the application, disk included,
// without the final backslash
function AppDir: string;
begin
  Result := ExtractFileDir(Application.ExeName);
  if LastChar(Result) = '\' then
    Result := Copy(Result, 1, Length(Result) - 1);
end;

// Creates a directory, even if the parent levels don't exist.
// With Delphi's CreateDir, the directory of "one level lower" must exist.
// Delphi's ForceDirectories also creates all in-between
// levels, but as it is a procedure, it does not return a value
// indicating if the directory is created or not.
// Return: TRUE if the directory is created.
function CreateDirForce(const NewDir: string): Boolean;
begin
  if not DirectoryExists(NewDir) then ForceDirectories(NewDir);
  Result := DirectoryExists(NewDir);
end;

// Return: TRUE if file is copied.
// If the destination file already exists:
//  - if Overwrite is TRUE, the destination is overwritten
//  - if Overwrite is FALSE, the function fails (i.e., it returns FALSE)
// If necessary, a new directory is created,
// even if the parent directories don't exit yet.
function FileCopy(const Source, Dest: string;
  const Overwrite: Boolean): Boolean;
begin
  Result := CreateDirForce(ExtractFileDir(Dest));
  if Result then
    Result := CopyFile(PChar(Source), PChar(Dest), (not Overwrite));
end;

// Move a file from Source to Dest.
// Return: fmovOK if the file is successfully moved,
// fmovNOTDELETED if copied but original not deleted,
// fmovFAILED if even the copy was not made.
// If the destination file already exists:
//  - if Overwrite is TRUE, the destination is overwritten
//  - if Overwrite is FALSE, the function fails (i.e., it returns FALSE)
// If necessary, a new directory is created,
// even if the parent directories don't exit yet.
function FileMove(const Source, Dest: string;
  const Overwrite: Boolean): TMoveResult;
begin
  if FileCopy(Source, Dest, Overwrite) then begin
    if DeleteFile(Source) then Result := fmovOK
    else Result := fmovNOTDELETED;
  end
  else
    Result := fmovFAIL;
end;

// Return: date and time of creation/ lastmodification of a file.
// Returns 0 if the file doesn't exist
function FileDateTime(const TheFile: string): TDateTime;
var
  DateTimeStamp: integer;
begin
  // FileAge() returns the date/time stamp of a file.
  // The returned value is an integer number;
  // it returns -1 if file not found
  DateTimeStamp := FileAge(TheFile);
  // If the file exists, convert DateTimeStamp to TDateTime format
  // If it doesn't exist, return 0 (this is 30 dec. 1899)
  if DateTimeStamp >= 0 then
    Result := FileDateToDateTime(DateTimeStamp)
  else
    Result := 0;
end;

// Return: date and time of creation/ lastmodification of a file
// as a string (short date format).
// Returns empty string if the file doesn't exist
function FileDateTimeStr(const TheFile: string): string;
var
  DT: TDateTime;
begin
  DT := FileDateTime(TheFile);
  // FileDateTime returns 0 if file not found
  if DT > 0 then Result := DateTimeToStr(DT)
  else Result := '';
end;


//--------- String -----

// Return: TRUE if string S only contains spaces
// and/or control characters
function IsEmptyStr(const S: string): Boolean;
begin
  // Trim() returns a string without leading and
  // trailing spaces nor control characters
  Result := Length(Trim(S)) = 0;
end;

// Return: TRUE if SubStr is contained in string S
function IsInStr(const SubStr, S: string): Boolean;
begin
  Result := Pos(SubStr, S) > 0;
end;

// Return: last character of a string
function LastChar(const S: string): Char;
begin
  Result := S[Length(S)];
end;

// Return: the leftmost LEN characters
function LeftStr(const S: string; const Len: integer): string;
begin
  Result := Copy(S, 1, Len);
end;

// Return: TRUE if SubStr is not contained in string S
function NotInStr(const SubStr, S: string): Boolean;
begin
  Result := Pos(SubStr, S) = 0;
end;

// Return: the rightmost LEN characters
function RightStr(const S: string; const Len: integer): string;
begin
  Result := Copy(S, Length(S) + 1 - Len, Len);
end;

// Return: a string of length Len, padded left whith characters "Ch"
// If Len <= the length of S, S is returned
function PadLeft(const S: string; const Len: integer; const Ch: char): string;
begin
  Result := S;
  while Length(Result) < Len do Result := Ch + Result;
end;

// Return: a string of length Len, padded right whith characters "Ch"
// If Len <= the length of S, S is returned
function PadRight(const S: string; const Len: integer; const Ch: char): string;
begin
  Result := S;
  while Length(Result) < Len do Result := Result + Ch;
end;

// Return: the remainder of a string S when the first word
// is parsed from it.
function ParseFirstRemoved(const S: string; const Delimiters: string): string;
begin
  Result := S;
  // Strip delimiters at beginning
  while (Length(Result) > 0) and
        (IsInStr(Result[1], Delimiters))  do
    System.Delete(Result, 1, 1);
  // Strip first word, until a delimiter or until the end
  while (Length(Result) > 0) and
        (NotInStr(Result[1], Delimiters))  do
    System.Delete(Result, 1, 1);
  // Finally, strip delimiters from the remainder
  while (Length(Result) > 0) and
        (IsInStr(Result[1], Delimiters))  do
    System.Delete(Result, 1, 1);
end;

// Return: the first word parsed from a string S
function ParseGetFirst(const S: string; const Delimiters: string): string;
var
  i: integer;
begin
  Result := '';
  i := 1;
  // Skip delimiters if there are any at the beginning
  while i <= Length(S) do
    if NotInStr(S[i], Delimiters) then break
    else inc(i);
  // Copy characters until a delimiter is found, or until the end
  while (i <= Length(S)) and
        (NotInStr(S[i], Delimiters)) do begin
    Result := Result + S[i];
    inc(i);
  end;
end;

// Return: the last word parsed from a string S
function ParseGetLast(const S: string; const Delimiters: string): string;
var
  i: integer;
begin
  Result := '';
  i := Length(S);
  // Skip delimiters if there are any at the end
  while i > 0 do
    if NotInStr(S[i], Delimiters) then break
    else dec(i);
  // Copy characters until a delimiter is found, or until the beginning
  while (i > 0) and
        (NotInStr(S[i], Delimiters)) do begin
    Result := S[i] + Result;
    dec(i);
  end;
end;

// Return: a stringlist with the words parsed from string S
function ParseToList(S: string; const Delimiters: string): TStringList;
begin
  Result := TStringList.Create;
  while Length(S) > 0 do begin
    Result.Add(ParseGetFirst(S, Delimiters));
    S := ParseFirstRemoved(S, Delimiters);
  end;
end;

// Return: a string of length Len, consisting of repeated strings S.
// If Len <= 0 then an empty string is returned.
function RepliStr(const S: string; const Len: integer): string;
var
  i: integer;
begin
  Result := '';
  for i := 1 to Len do Result := Result + S;
end;

end.
